'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
// The module 'vscode' contains the VS Code extensibility API
// Import the module and reference it with the alias vscode in your code below
const vscode = require("vscode");
var path = "";
var server = undefined;
var fs = require("fs");
var net = require('net');
var outData = {};
var calcHelpSkt = undefined;
var ckBoxCtrlSkt = undefined;
var dfRetSkt = undefined;
var toolName = "";
var outChannel;
outData["Id"] = 'VSC';
// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
function activate(context) {
    // Use the console to output diagnostic information (console.log) and errors (console.error)
    // This line of code will only be executed once when your extension is activated
    console.log('Congratulations, your extension "calc-tools-server-ext" is now active!');
    outChannel = vscode.window.createOutputChannel("CalcToolsSrv_DEBUG");
    try {
        //console.log('os.userInfo().username: ' + os.userInfo().username);
        //console.log('Executing Calc Helper Extension!');
        server = net.createServer({
            keepAlive: true
        });
        server.listen(0, function () {
            console.log('Listening on port ' + server.address().port);
            outChannel.appendLine('Listening on port ' + server.address().port);
            outChannel.show();
            // Write the port number to .calctools file:
            if (vscode.workspace.workspaceFolders !== undefined) {
                // vscode.window.showErrorMessage('Workspce folders present.');
                // outChannel.appendLine('Workspce folders present.' );
                // outChannel.show();
                // Could not use 'Users' folder and work off from that die to differences
                // in people's computers user names, for example, 'kames' and 'kames.CORP'
                // are on same computer, using userName from Javascript and C# give 'kames'
                // but 'kames' is not accessible on Kevin's own computer!
                path = 'C:\\temp\\CalcHelper';
                if (!fs.existsSync(path)) {
                    fs.mkdirSync(path);
                }
                path += '\\.calctools';
                // console.log(`Writing port #${server.address().port} to ${path}`);
                // outChannel.appendLine(`Writing port #${server.address().port} to ${path}` );
                // outChannel.show();
                var port = server.address().port;
                console.log(`Writing port #${port} to ${path}`);
                outChannel.appendLine(`Writing port #${port} to ${path}`);
                outChannel.show();
                // fs.writeFileSync(path, port, function (err: any) {
                // 	if (err) {
                // 		// .calctools file write failed.
                // 		vscode.window.showErrorMessage(`.calctools FILE WRITE ERROR: ${err.message}`);
                // 		outChannel.appendLine("ERROR: " + `.calctools FILE WRITE ERROR: ${err.message}` );
                // 		outChannel.show();
                // 	} else {
                // 		// Success.
                // 		console.log('.calctools port file has been written!');
                // 		outChannel.appendLine('.calctools port file has been written!' );
                // 		outChannel.show();
                // 	}
                // });
                try {
                    fs.writeFileSync(path, `${port}`, {
                        encoding: "utf8",
                        flag: "w+",
                        mode: 0o666
                    });
                }
                catch (err) {
                    vscode.window.showErrorMessage(`.calctools FILE WRITE ERROR: ${err.message}`);
                    outChannel.appendLine("ERROR: " + `.calctools FILE WRITE ERROR: ${err.message}`);
                    outChannel.show();
                }
                //outChannel.appendLine('After write port file.' );
                //outChannel.show();
                /*
                // Testing setting cursor to 'var calc_controlled = [' area in a document's script file.
                // Need to polish off in terms of error reporting and checking for a .js file.
                // The biggest problem is this extension and the calcext fire at the same time, so sometimes
                // the search works and sometimes not.
                let editor = vscode.window.activeTextEditor;
                if (editor) {

                    // Trying to work around the starting from SB .stf port file issue:
                    // This does not seem to work at all. Maybe a timing thing again...
                    //vscode.commands.executeCommand("workbench.action.output.toggleOutput");
                    //vscode.commands.executeCommand("workbench.debug.action.toggleRepl");

                    vscode.window.showTextDocument;
                    vscode.commands.executeCommand("actions.find").then(res => {
                    vscode.commands.executeCommand("editor.action.clipboardCopyAction", "var calc_controlled = [");
                    vscode.commands.executeCommand("editor.action.clipboardPasteAction");
                    vscode.commands.executeCommand("closeFindWidget");
                    }, err => {
                        console.error(err);
                    });
                }
                */
            }
            else {
                outChannel.appendLine('Workspace folders empty.');
                outChannel.show();
            }
        });
        server.on("connection", (socket) => {
            // Add a 'error' event handler to this instance of server
            socket.on('error', function (e) {
                //console.log(`ERROR: ${e.message}`);
                vscode.window.showErrorMessage(`ERROR: ${e.message}`);
                outChannel.appendLine(`ERROR: ${e.message}`);
                outChannel.show();
            });
            // Add a client 'close' event handler to this instance of server
            socket.on('end', () => {
                //console.log('CLOSED: ' + socket.remoteAddress +' '+ socket.remotePort);
                // Which Calc Tool is disconnecting?
                switch (socket) {
                    case calcHelpSkt: {
                        toolName = "Calc Helper";
                        calcHelpSkt = undefined;
                        // Inform any other connected tools:
                        if (ckBoxCtrlSkt !== undefined) {
                            outData["Command"] = "Disconnect-CH";
                            ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        if (dfRetSkt !== undefined) {
                            outData["Command"] = "Disconnect-CH";
                            dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case ckBoxCtrlSkt: {
                        toolName = "Check Box Control";
                        ckBoxCtrlSkt = undefined;
                        // Inform Calc Helper if connected:
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "Disconnect-CBC";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case dfRetSkt: {
                        toolName = "Dataflow Retriever";
                        dfRetSkt = undefined;
                        // Inform Calc Helper if connected:
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "Disconnect-DFR";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                }
                if (toolName.length > 0) {
                    vscode.window.showInformationMessage(`${toolName} is disconnecting.`);
                    //console.log(`${toolName} is disconnecting.`);
                }
            });
            // Add a 'data' event handler to this instance of server:
            socket.on('data', function (buf) {
                // console.log('Data received...');
                // outChannel.appendLine('Data received...' );
                // outChannel.show();
                //console.log(buf);
                var length = buf.readInt32LE();
                var buffer = Buffer.alloc(length);
                buf.copy(buffer, 0, 4);
                var str = buffer.toString();
                var json = JSON.parse(str);
                switch (json["Command"]) {
                    case "Connect": {
                        // Which Calc Tool is connecting?.
                        // console.info(`Client connecting [id=${json["Id"]}]`);
                        toolName = "";
                        switch (json["Id"]) {
                            case "CH": {
                                calcHelpSkt = socket;
                                toolName = "Calc Helper";
                                break;
                            }
                            case "CBC": {
                                ckBoxCtrlSkt = socket;
                                toolName = "Check Box Control";
                                break;
                            }
                            case "DFR": {
                                dfRetSkt = socket;
                                toolName = "Dataflow Retriever";
                                break;
                            }
                            default: {
                                vscode.window.showErrorMessage("Connect message received from unknown client.");
                                break;
                            }
                        }
                        if (toolName.length > 0) {
                            //console.log(`${toolName} Has CONNECTED: ${socket.remoteAddress}:${socket.remotePort}`);
                            vscode.window.showInformationMessage(`${toolName} Has CONNECTED`);
                            // Now send back a "ConnectAck" command:
                            outData["Command"] = "ConnectAck";
                            socket.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "BoxSelDOC": {
                        // Calc Helper box Selection(s) for the current document being edited.
                        let editor = vscode.window.activeTextEditor;
                        if (!editor) {
                            return;
                        }
                        // Insert at cursor, go to next tab stop if in a snippet.
                        vscode.window.showTextDocument(editor.document);
                        vscode.commands.executeCommand("editor.action.clipboardPasteAction").then(res => {
                            vscode.commands.executeCommand("jumpToNextSnippetPlaceholder");
                        }, err => {
                            //console.log(`ERROR: ${err.message}`);
                            vscode.window.showErrorMessage(`ERROR: ${err.message}`);
                        });
                        break;
                    }
                    case "GotoCode": {
                        // Calc Helper: Goto [i.e.find] this code.
                        // We keep the Find Widget open.
                        let editor = vscode.window.activeTextEditor;
                        if (!editor) {
                            return;
                        }
                        vscode.commands.executeCommand("actions.find").then(res => {
                            vscode.commands.executeCommand("editor.action.clipboardPasteAction");
                        }, err => {
                            vscode.window.showErrorMessage(`ERROR: ${err.message}`);
                        });
                        break;
                    }
                    case "GotoCodeCloseFind": {
                        // Calc Helper: Goto [i.e.find] this code.
                        // The Find Widget is closed.
                        let editor = vscode.window.activeTextEditor;
                        if (!editor) {
                            return;
                        }
                        if (json["Data"] === "ToFirstLine") {
                            // This scrolls to the first line.
                            let range = editor.document.lineAt(0).range;
                            editor.selection = new vscode.Selection(range.start, range.end);
                            editor.revealRange(range);
                        }
                        vscode.commands.executeCommand("actions.find").then(res => {
                            vscode.commands.executeCommand("editor.action.clipboardPasteAction");
                            vscode.commands.executeCommand("closeFindWidget");
                        }, err => {
                            vscode.window.showErrorMessage(`ERROR: ${err.message}`);
                        });
                        break;
                    }
                    case "CHBoxSelOffCBC": {
                        // Calc Helper is telling CBC to revert back to it's box list for selections.
                        // Why? The user might have decided to no longer send check box selections
                        // to CBC, or Calc Helper is shutting down, or the user wants a different tool
                        // to receive box selections from Calc Helper.
                        // console.log("Calc Helper is telling CBC to revert back to it's box list for selections.");
                        if (ckBoxCtrlSkt !== undefined) {
                            outData["Command"] = "CHBoxSelOffCBC";
                            ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "CBCBoxSets": {
                        // Check Box Control is sending Calc Helper the contents of the
                        // Checkbox Sets list. This is sent when the list's contents
                        // are changed.
                        // console.log("Checkbox Control box sets will be sent to Calc Helper.
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "CBCBoxSets";
                            outData["FileName"] = json["FileName"];
                            outData["Data"] = json["Data"];
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "CBCBoxUnSel": {
                        // Check Box Control is sending Calc Helper boxes to unselect.
                        // Could be that CH sent boxes to CBC but file names differed,
                        // or a check box was already in the CBC box list. Just a
                        // visual nicety - maybe overboard.
                        // console.log("Checkbox Control sent box(es) to CH to unselect.
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "CBCBoxUnSel";
                            outData["Data"] = json["Data"];
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "CBCBoxSelOff": {
                        // Check Box Control no longer wants check box selections from Calc
                        // Helper. Either the user is shutting down Check Box Control
                        // or has switched the box selection mode back to the box list, or
                        // the last box selection message had a Calc Helper file name different
                        // from the Check Box Control file name [probably not likely] or the
                        // user switched Checkbox Control to a new file name.
                        // console.log("Checkbox Control now gets check box selection(s) from it's box list");
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "CBCBoxSelOff";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        // Echo this command back to Check Box Control so it can keep
                        // receiving mesages from Calc Helper. Probably a better way...
                        outData["Command"] = "CBCBoxSelOff";
                        ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        break;
                    }
                    case "CBCBoxSel": {
                        // Check Box Control is requesting Calc Helper for check box selections.
                        // Calc Helper needs to be linked to the same file. CBC is sending that along.
                        // console.log("Check Box Control wants check box selection(s) from Calc Helper");
                        if (calcHelpSkt === undefined) {
                            outData["Command"] = "CBCBoxSelNack";
                            ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        else {
                            outData["Command"] = "CBCBoxSel";
                            outData["FileName"] = json["FileName"];
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "CBCBoxSelFileNack": {
                        // Calc Helper is denying the CBC request for check box selections due to
                        // different linked file names between the two tools. The file name below
                        // is the one Calc Helper is linked to.
                        // console.log("Calc Helper denys CBC check box selection request\n due to different files.");
                        if (ckBoxCtrlSkt !== undefined) {
                            outData["Command"] = "CBCBoxSelFileNack";
                            outData["FileName"] = json["FileName"];
                            ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "CBCBoxSelAck": {
                        // Calc Helper is acknowledging the CBC check box selections request:
                        // console.log("CH acknowledges the CBC check box selection(s) request.");
                        if (ckBoxCtrlSkt === undefined) {
                            outData["Command"] = "CBCBoxSelNack";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        else {
                            outData["Command"] = "CBCBoxSelAck";
                            ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "BoxSelCBC": {
                        // Calc Helper box selection(s) for Check Box Control.
                        // The file name below is the one Calc Helper is linked to.
                        // Each time Check Box Control will check to see that both
                        // tools are linked to the same file name.
                        // console.log("Box selection(s) for Check Box Control");
                        if (ckBoxCtrlSkt === undefined) {
                            // I don't think this will happen....
                            outData["Command"] = "BoxSelCBCNack";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        else {
                            outData["Command"] = "BoxSelCBC";
                            outData["Data"] = json["Data"];
                            outData["FileName"] = json["FileName"];
                            ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "CHBoxSelOffDFR": {
                        // Calc Helper is telling DFR to revert back to it's box list for selections.
                        // console.log("Calc Helper is telling DFR to revert back to it's box list for selections.");
                        if (dfRetSkt !== undefined) {
                            outData["Command"] = "CHBoxSelOffDFR";
                            dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "DFRBoxUnSel": {
                        // Dataflow Retriever is sending Calc Helper boxes to unselect.
                        // Could be that CH sent boxes to DFR but file names differed,
                        // or some other reason.
                        // console.log("Dataflow Retriever sent box(es) to CH to unselect.
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "DFRBoxUnSel";
                            outData["Data"] = json["Data"];
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "DFRBoxSelOff": {
                        // Dataflow Retriever has changed box selection mode back to
                        // it's box list.
                        // console.log("Dataflow Retriever now gets box selection(s) from it's box list");
                        if (calcHelpSkt !== undefined) {
                            outData["Command"] = "DFRBoxSelOff";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "DFRBoxSel": {
                        // Dataflow Retriever is requesting Calc Helper for box selections.
                        // Calc Helper needs to be linked to the same file. DFR is sending that along.
                        // console.log("Dataflow Retriever wants box selection(s) from Calc Helper");
                        if (calcHelpSkt === undefined) {
                            outData["Command"] = "DFRBoxSelNack";
                            dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        else {
                            outData["Command"] = "DFRBoxSel";
                            outData["FileName"] = json["FileName"];
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "DFRBoxSelFileNack": {
                        // Calc Helper is denying the DFR request for box selections due to
                        // different linked file names between the two tools. The file name below
                        // is the one Calc Helper is linked to.
                        // console.log("Calc Helper denys DFR box selection request\n due to different files.");
                        if (dfRetSkt !== undefined) {
                            outData["Command"] = "DFRBoxSelFileNack";
                            outData["FileName"] = json["FileName"];
                            dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "DFRBoxSelAck": {
                        // Calc Helper is acknowledging the DFR box selections request:
                        // console.log("CH acknowledges the DFR box selection(s) request.");
                        if (dfRetSkt === undefined) {
                            outData["Command"] = "DFRBoxSelNack";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        else {
                            outData["Command"] = "DFRBoxSelAck";
                            dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "BoxSelDFR": {
                        // Calc Helper box selection(s) for Dataflow Retriever:
                        // The file name below is the one Calc Helper is linked to.
                        // console.log("Box selection(s) for Dataflow Retriever");
                        if (dfRetSkt === undefined) {
                            // I don't think this will happen....
                            outData["Command"] = "BoxSelDFRNack";
                            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        else {
                            outData["Command"] = "BoxSelDFR";
                            outData["Data"] = json["Data"];
                            outData["FileName"] = json["FileName"];
                            dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
                        }
                        break;
                    }
                    case "InsertSnippet": {
                        let editor = vscode.window.activeTextEditor;
                        if (!editor) {
                            return;
                        }
                        vscode.commands.executeCommand("editor.action.insertSnippet", { name: json["Data"] });
                        break;
                    }
                }
            });
        });
    }
    catch (err) {
        console.log(err.message);
        vscode.window.showErrorMessage(`ERROR: ${err.message}`);
        outChannel.appendLine(`ERROR: ${err.message}`);
        outChannel.show();
    }
    vscode.workspace.onDidOpenTextDocument((document) => {
        //console.log("onDidOpenTextDocument: " + document.fileName);
        if (calcHelpSkt !== undefined) {
            outData["Command"] = "OpenDoc";
            outData["FileName"] = document.fileName;
            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
        }
    });
    vscode.workspace.onDidCloseTextDocument((document) => {
        //console.log("onDidCloseTextDocument: " + document.fileName);
        if (calcHelpSkt !== undefined) {
            outData["Command"] = "CloseDoc";
            outData["FileName"] = document.fileName;
            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
        }
    });
    vscode.workspace.onDidSaveTextDocument((document) => {
        // console.log("onDidSaveTextDocument " + document.fileName");
        if (calcHelpSkt !== undefined) {
            outData["Command"] = "SaveDoc";
            outData["FileName"] = document.fileName;
            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
        }
    });
    vscode.window.onDidChangeActiveTextEditor((textEditor) => {
        if (calcHelpSkt !== undefined) {
            outData["Command"] = "ActiveTextEditorDelta";
            outData["FileName"] = (textEditor === undefined ? "undefined" : textEditor.document.fileName);
            calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
        }
    });
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() {
    outData["Command"] = "Disconnecting";
    outData["FileName"] = "";
    // We will shutdown Calc Helper.
    if (calcHelpSkt !== undefined) {
        calcHelpSkt.write(MessageToBuffer(JSON.stringify(outData)));
    }
    if (ckBoxCtrlSkt !== undefined) {
        ckBoxCtrlSkt.write(MessageToBuffer(JSON.stringify(outData)));
    }
    if (dfRetSkt !== undefined) {
        dfRetSkt.write(MessageToBuffer(JSON.stringify(outData)));
    }
    server.close();
}
exports.deactivate = deactivate;
function MessageToBuffer(msg) {
    var buffer = Buffer.alloc(msg.length + 4);
    buffer.writeInt32LE(msg.length, 0);
    buffer.write(msg, 4);
    return buffer;
}
//# sourceMappingURL=extension.js.map